/*	Public domain	*/

#ifndef _AGAR_PROP_H_
#define _AGAR_PROP_H_

#ifdef _AGAR_INTERNAL
#include <config/have_long_double.h>
#else
#include <agar/config/have_long_double.h>
#endif

#include "begin_code.h"

#define AG_PROP_PATH_MAX	(AG_OBJECT_PATH_MAX+1+AG_PROP_KEY_MAX)
#define AG_PROP_KEY_MAX		64
/* #define AG_PROP_STRING_LIMIT	65536 */

enum ag_prop_type {
	AG_PROP_UINT		= 0,
	AG_PROP_INT		= 1,
	AG_PROP_UINT8		= 2,
	AG_PROP_SINT8		= 3,
	AG_PROP_UINT16		= 4,
	AG_PROP_SINT16		= 5,
	AG_PROP_UINT32		= 6,
	AG_PROP_SINT32		= 7,
	AG_PROP_UINT64		= 8,	/* Optional */
	AG_PROP_SINT64		= 9,	/* Optional */
	AG_PROP_FLOAT		= 10,
	AG_PROP_DOUBLE		= 11,
	AG_PROP_LONG_DOUBLE	= 12,	/* Optional */
	AG_PROP_STRING		= 13,
	AG_PROP_POINTER		= 14,
	AG_PROP_BOOL		= 15,
	AG_PROP_EXT_FIRST	= 100,		/* Official extensions */
	AG_PROP_EXT_LAST	= 10000,
	AG_PROP_PVT_START	= 10001,	/* Application extensions */
	AG_PROP_PVT_END		= 65535
};

#if 0
struct ag_prop_ops {
	enum ag_prop_type type;
	const char *descr;
	void (*set_val)(void *obj, AG_Prop *prop, void *arg);
	void *(*get_val)(void *obj, AG_Prop *prop);
	void (*print_val)(char *buf, size_t len, void *obj, AG_Prop *prop);
	int (*compare)(AG_Prop *, AG_Prop *);
	void *(*load)(void *obj, AG_Prop *, AG_DataSource *);
	void (*save)(void *obj, AG_Prop *, AG_DataSource *);
} AG_PropOps;
#endif

typedef struct ag_prop {
	char key[AG_PROP_KEY_MAX];
	int type;
	union {
		unsigned u;
		int	 i;
		Uint8	 u8;
		Sint8	 s8;
		Uint16	 u16;
		Sint16	 s16;
		Uint32	 u32;
		Sint32	 s32;
#ifdef HAVE_64BIT
		Uint64   u64;
		Sint64   s64;
#endif
		float	 f;
		double	 d;
#ifdef HAVE_LONG_DOUBLE
		long double ld;
#endif
		char	*s;
		void	*p;
	} data;
	union {
		Uint	(*wUint)(void *, struct ag_prop *, Uint);
		int	(*wInt)(void *, struct ag_prop *, int);
		int	(*wBool)(void *, struct ag_prop *, int);
		Uint8	(*wUint8)(void *, struct ag_prop *, Uint8);
		Sint8	(*wSint8)(void *, struct ag_prop *, Sint8);
		Uint16	(*wUint16)(void *, struct ag_prop *, Uint16);
		Sint16	(*wSint16)(void *, struct ag_prop *, Sint16);
		Uint32	(*wUint32)(void *, struct ag_prop *, Uint32);
		Sint32	(*wSint32)(void *, struct ag_prop *, Sint32);
#ifdef HAVE_64BIT
		Uint64	(*wUint64)(void *, struct ag_prop *, Uint64);
		Sint64	(*wSint64)(void *, struct ag_prop *, Sint64);
#endif
		float	(*wFloat)(void *, struct ag_prop *, float);
		double	(*wDouble)(void *, struct ag_prop *, double);
#ifdef HAVE_LONG_DOUBLE
		long double (*wLongDouble)(void *, struct ag_prop *,
		                           long double);
#endif
		char	*(*wString)(void *, struct ag_prop *, char *);
		void	*(*wPointer)(void *, struct ag_prop *, void *);
	} writeFn;
	union {
		Uint	(*rUint)(void *, struct ag_prop *);
		int	(*rInt)(void *, struct ag_prop *);
		int	(*rBool)(void *, struct ag_prop *);
		Uint8	(*rUint8)(void *, struct ag_prop *);
		Sint8	(*rSint8)(void *, struct ag_prop *);
		Uint16	(*rUint16)(void *, struct ag_prop *);
		Sint16	(*rSint16)(void *, struct ag_prop *);
		Uint32	(*rUint32)(void *, struct ag_prop *);
		Sint32	(*rSint32)(void *, struct ag_prop *);
#ifdef HAVE_64BIT
		Uint64	(*rUint64)(void *, struct ag_prop *);
		Sint64	(*rSint64)(void *, struct ag_prop *);
#endif
		float	(*rFloat)(void *, struct ag_prop *);
		double	(*rDouble)(void *, struct ag_prop *);
#ifdef HAVE_LONG_DOUBLE
		long double (*rLongDouble)(void *, struct ag_prop *);
#endif
		char	*(*rString)(void *, struct ag_prop *);
		void	*(*rPointer)(void *, struct ag_prop *);
	} readFn;
	AG_TAILQ_ENTRY(ag_prop) props;
} AG_Prop;

#define AG_SetUintWrFn(prop,fn) (prop)->writeFn.wUint = (fn)
#define AG_SetIntWrFn(prop,fn) (prop)->writeFn.wInt = (fn)
#define AG_SetBoolWrFn(prop,fn) (prop)->writeFn.wBool = (fn)
#define AG_SetUint8WrFn(prop,fn) (prop)->writeFn.wUint8 = (fn)
#define AG_SetSint8WrFn(prop,fn) (prop)->writeFn.wSint8 = (fn)
#define AG_SetUint16WrFn(prop,fn) (prop)->writeFn.wUint16 = (fn)
#define AG_SetSint16WrFn(prop,fn) (prop)->writeFn.wSint16 = (fn)
#define AG_SetUint32WrFn(prop,fn) (prop)->writeFn.wUint32 = (fn)
#define AG_SetSint32WrFn(prop,fn) (prop)->writeFn.wSint32 = (fn)
#define AG_SetUint64WrFn(prop,fn) (prop)->writeFn.wUint64 = (fn)
#define AG_SetSint64WrFn(prop,fn) (prop)->writeFn.wSint64 = (fn)
#define AG_SetFloatWrFn(prop,fn) (prop)->writeFn.wFloat = (fn)
#define AG_SetDoubleWrFn(prop,fn) (prop)->writeFn.wDouble = (fn)
#define AG_SetLongDoubleWrFn(prop,fn) (prop)->writeFn.wLongDouble = (fn)
#define AG_SetStringWrFn(prop,fn) (prop)->writeFn.wString = (fn)
#define AG_SetPointerWrFn(prop,fn) (prop)->writeFn.wPointer = (fn)

#define AG_SetUintRdFn(prop,fn) (prop)->readFn.rUint = (fn)
#define AG_SetIntRdFn(prop,fn) (prop)->readFn.rInt = (fn)
#define AG_SetBoolRdFn(prop,fn) (prop)->readFn.rBool = (fn)
#define AG_SetUint8RdFn(prop,fn) (prop)->readFn.rUint8 = (fn)
#define AG_SetSint8RdFn(prop,fn) (prop)->readFn.rSint8 = (fn)
#define AG_SetUint16RdFn(prop,fn) (prop)->readFn.rUint16 = (fn)
#define AG_SetSint16RdFn(prop,fn) (prop)->readFn.rSint16 = (fn)
#define AG_SetUint32RdFn(prop,fn) (prop)->readFn.rUint32 = (fn)
#define AG_SetSint32RdFn(prop,fn) (prop)->readFn.rSint32 = (fn)
#define AG_SetUint64RdFn(prop,fn) (prop)->readFn.rUint64 = (fn)
#define AG_SetSint64RdFn(prop,fn) (prop)->readFn.rSint64 = (fn)
#define AG_SetFloatRdFn(prop,fn) (prop)->readFn.rFloat = (fn)
#define AG_SetDoubleRdFn(prop,fn) (prop)->readFn.rDouble = (fn)
#define AG_SetLongDoubleRdFn(prop,fn) (prop)->readFn.rLongDouble = (fn)
#define AG_SetStringRdFn(prop,fn) (prop)->readFn.rString = (fn)
#define AG_SetPointerRdFn(prop,fn) (prop)->readFn.rPointer = (fn)

__BEGIN_DECLS
int	 AG_PropLoad(void *, AG_DataSource *);
int	 AG_PropSave(void *, AG_DataSource *);
void	 AG_PropDestroy(AG_Prop *);
AG_Prop	*AG_CopyProp(const AG_Prop *);
AG_Prop	*AG_SetProp(void *, const char *, enum ag_prop_type, ...);

AG_Prop	 *AG_GetProp(void *, const char *, int, void *);
AG_Prop	 *AG_FindProp(void *, const char *, int, void *);
void	  AG_PropPrint(char *, size_t, void *, const char *)
		               BOUNDED_ATTRIBUTE(__string__, 1, 2);
int	  AG_PropPath(char *, size_t, void *, const char *)
		              BOUNDED_ATTRIBUTE(__string__, 1, 2);
size_t	  AG_StringCopy(void *, const char *, char *, size_t);
size_t	  AG_FindStringCopy(void *, const char *, char *, size_t);
AG_Prop  *AG_SetString(void *, const char *, const char *, ...);

static __inline__ AG_Prop *AG_SetUint(void *ob, const char *key, Uint i) {
	return (AG_SetProp(ob, key, AG_PROP_UINT, i));
}
static __inline__ AG_Prop *AG_SetInt(void *ob, const char *key, int i) {
	return (AG_SetProp(ob, key, AG_PROP_INT, i));
}
static __inline__ AG_Prop *AG_SetUint8(void *ob, const char *key, Uint8 i) {
	return (AG_SetProp(ob, key, AG_PROP_UINT8, i));
}
static __inline__ AG_Prop *AG_SetSint8(void *ob, const char *key, Sint8 i) {
	return (AG_SetProp(ob, key, AG_PROP_SINT8, i));
}
static __inline__ AG_Prop *AG_SetUint16(void *ob, const char *key, Uint16 i) {
	return (AG_SetProp(ob, key, AG_PROP_UINT16, i));
}
static __inline__ AG_Prop *AG_SetSint16(void *ob, const char *key, Sint16 i) {
	return (AG_SetProp(ob, key, AG_PROP_SINT16, i));
}
static __inline__ AG_Prop *AG_SetUint32(void *ob, const char *key, Uint32 i) {
	return (AG_SetProp(ob, key, AG_PROP_UINT32, i));
}
static __inline__ AG_Prop *AG_SetSint32(void *ob, const char *key, Sint32 i) {
	return (AG_SetProp(ob, key, AG_PROP_SINT32, i));
}

#ifdef HAVE_64BIT
static __inline__ AG_Prop *AG_SetUint64(void *ob, const char *key, Uint64 i) {
	return (AG_SetProp(ob, key, AG_PROP_UINT64, i));
}
static __inline__ AG_Prop *AG_SetSint64(void *ob, const char *key, Sint64 i) {
	return (AG_SetProp(ob, key, AG_PROP_SINT64, i));
}
#endif /* HAVE_64BIT */

static __inline__ AG_Prop *AG_SetFloat(void *ob, const char *key, float f) {
	return (AG_SetProp(ob, key, AG_PROP_FLOAT, f));
}
static __inline__ AG_Prop *AG_SetDouble(void *ob, const char *key, double d) {
	return (AG_SetProp(ob, key, AG_PROP_DOUBLE, d));
}

#ifdef HAVE_LONG_DOUBLE
static __inline__ AG_Prop *
AG_SetLongDouble(void *ob, const char *key, long double d)
{
	return (AG_SetProp(ob, key, AG_PROP_LONG_DOUBLE, d));
}
#endif /* HAVE_LONG_DOUBLE */

static __inline__ AG_Prop *AG_SetPointer(void *ob, const char *key, void *p) {
	return (AG_SetProp(ob, key, AG_PROP_POINTER, p));
}
static __inline__ AG_Prop *AG_SetBool(void *ob, const char *key, int i) {
	return (AG_SetProp(ob, key, AG_PROP_BOOL, i));
}
static __inline__ Uint AG_Uint(void *p, const char *key) {
	Uint i;
	if (AG_GetProp(p, key, AG_PROP_UINT, (Uint *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint AG_FindUint(void *vfsRoot, const char *key) {
	Uint i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_UINT, (Uint *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ int AG_Int(void *p, const char *key) {
	int i;
	if (AG_GetProp(p, key, AG_PROP_INT, (int *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ int AG_FindInt(void *vfsRoot, const char *key) {
	int i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_INT, (int *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ int AG_Bool(void *p, const char *key) {
	int i;
	if (AG_GetProp(p, key, AG_PROP_BOOL, (int *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ int AG_FindBool(void *vfsRoot, const char *key) {
	int i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_BOOL, (int *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint8 AG_Uint8(void *p, const char *key) {
	Uint8 i;
	if (AG_GetProp(p, key, AG_PROP_UINT8, (Uint8 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint8 AG_FindUint8(void *vfsRoot, const char *key) {
	Uint8 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_UINT8, (Uint8 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint8 AG_Sint8(void *p, const char *key) {
	Sint8 i;
	if (AG_GetProp(p, key, AG_PROP_SINT8, (Sint8 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint8 AG_FindSint8(void *vfsRoot, const char *key) {
	Sint8 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_SINT8, (Sint8 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint16 AG_Uint16(void *p, const char *key) {
	Uint16 i;
	if (AG_GetProp(p, key, AG_PROP_UINT16, (Uint16 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint16 AG_FindUint16(void *vfsRoot, const char *key) {
	Uint16 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_UINT16, (Uint16 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint16 AG_Sint16(void *p, const char *key) {
	Sint16 i;
	if (AG_GetProp(p, key, AG_PROP_SINT16, (Sint16 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint16 AG_FindSint16(void *vfsRoot, const char *key) {
	Sint16 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_SINT16, (Sint16 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint32 AG_Uint32(void *p, const char *key) {
	Uint32 i;
	if (AG_GetProp(p, key, AG_PROP_UINT32, (Uint32 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint32 AG_FindUint32(void *vfsRoot, const char *key) {
	Uint32 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_UINT32, (Uint32 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint32 AG_Sint32(void *p, const char *key) {
	Sint32 i;
	if (AG_GetProp(p, key, AG_PROP_SINT32, (Sint32 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint32 AG_FindSint32(void *vfsRoot, const char *key) {
	Sint32 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_SINT32, (Sint32 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}

#ifdef HAVE_64BIT
static __inline__ Uint64 AG_Uint64(void *p, const char *key) {
	Uint64 i;
	if (AG_GetProp(p, key, AG_PROP_UINT64, (Uint64 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint64 AG_Sint64(void *p, const char *key) {
	Sint64 i;
	if (AG_GetProp(p, key, AG_PROP_SINT64, (Sint64 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Uint64 AG_FindUint64(void *vfsRoot, const char *key) {
	Uint64 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_UINT64, (Uint64 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
static __inline__ Sint64 AG_FindSint64(void *vfsRoot, const char *key) {
	Sint64 i;
	if (AG_FindProp(vfsRoot, key, AG_PROP_SINT64, (Sint64 *)&i) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (i);
}
#endif /* HAVE_64BIT */

static __inline__ float AG_Float(void *p, const char *key) {
	float f;
	if (AG_GetProp(p, key, AG_PROP_FLOAT, (float *)&f) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (f);
}
static __inline__ float AG_FindFloat(void *vfsRoot, const char *key) {
	float f;
	if (AG_FindProp(vfsRoot, key, AG_PROP_FLOAT, (float *)&f) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (f);
}
static __inline__ double AG_Double(void *p, const char *key) {
	double d;
	if (AG_GetProp(p, key, AG_PROP_DOUBLE, (double *)&d) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (d);
}
static __inline__ double AG_FindDouble(void *vfsRoot, const char *key) {
	double d;
	if (AG_FindProp(vfsRoot, key, AG_PROP_DOUBLE, (double *)&d) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (d);
}

#ifdef HAVE_LONG_DOUBLE
static __inline__ long double AG_LongDouble(void *p, const char *key) {
	long double d;
	if (AG_GetProp(p, key, AG_PROP_LONG_DOUBLE, (long double *)&d)
	    == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (d);
}
static __inline__ long double AG_FindLongDouble(void *vfsRoot, const char *key){
	long double d;
	if (AG_FindProp(vfsRoot, key, AG_PROP_LONG_DOUBLE, (long double *)&d)
	    == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (d);
}
#endif /* HAVE_LONG_DOUBLE */

static __inline__ char *AG_String(void *p, const char *key) {
	char *s;
	if (AG_GetProp(p, key, AG_PROP_STRING, (char *)&s) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (s);
}
static __inline__ char *AG_FindString(void *vfsRoot, const char *key) {
	char *s;
	if (AG_FindProp(vfsRoot, key, AG_PROP_STRING, (char *)&s) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (s);
}
static __inline__ void * AG_Pointer(void *p, const char *key) {
	void *np;
	if (AG_GetProp(p, key, AG_PROP_POINTER, (void *)&np) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (np);
}
static __inline__ void * AG_FindPointer(void *vfsRoot, const char *key) {
	void *np;
	if (AG_FindProp(vfsRoot, key, AG_PROP_POINTER, (void *)&np) == NULL) {
		AG_FatalError("%s", AG_GetError());
	}
	return (np);
}
__END_DECLS

#include "close_code.h"
#endif /* _AGAR_PROP_H_ */
